from menpodetect.ffld2 import load_ffld2_frontal_face_detector
import menpo.io as mio

takeo = mio.import_builtin_asset.takeo_ppm()


def test_frontal_face_detector():
    takeo_copy = takeo.copy()
    ffld2_detector = load_ffld2_frontal_face_detector()
    pcs = ffld2_detector(takeo_copy, threshold=2)
    assert len(pcs) == 1
    assert takeo_copy.n_channels == 3
    assert takeo_copy.landmarks['ffld2_0'].n_points == 4


def test_frontal_face_detector_rgb():
    takeo_copy = takeo.copy()
    assert takeo_copy.n_channels == 3
    ffld2_detector = load_ffld2_frontal_face_detector()
    pcs = ffld2_detector(takeo_copy, greyscale=False, threshold=2)
    assert len(pcs) == 1
    assert takeo_copy.n_channels == 3
    assert takeo_copy.landmarks['ffld2_0'].n_points == 4



from menpodetect.pico import load_pico_frontal_face_detector
import menpo.io as mio

takeo = mio.import_builtin_asset.takeo_ppm()


def test_frontal_face_detector():
    takeo_copy = takeo.copy()
    pico_detector = load_pico_frontal_face_detector()
    pcs = pico_detector(takeo_copy)
    assert len(pcs) == 1
    assert takeo_copy.n_channels == 3
    assert takeo_copy.landmarks['pico_0'].n_points == 4



from __future__ import division
from functools import partial
from pathlib import Path

from menpo.base import MenpoMissingDependencyError

try:
    import cv2
except ImportError:
    raise MenpoMissingDependencyError('opencv')

from menpodetect.detect import detect
from menpodetect.compatibility import STRING_TYPES
from .conversion import (pointgraph_from_rect, opencv_frontal_face_path,
                         opencv_profile_face_path, opencv_eye_path)


def _get_default_flags():
    version = cv2.__version__.split('.')[0]
    if version == '2':
        return cv2.cv.CV_HAAR_SCALE_IMAGE
    elif version == '3':
        return cv2.CASCADE_SCALE_IMAGE


class _opencv_detect(object):
    r"""
    A utility callable that allows the caching of an opencv detector.

    This callable is important for presenting the correct parameters to the
    user. It also marshalls the return type of the detector back to
    menpo.shape.PointDirectedGraph.

    Parameters
    ----------
    model : `Path` or `str` or `opencv.CascadeClassifier`
        Either a path to an `opencv.CascadeClassifier` or the detector itself.

    Raises
    ------
    ValueError
        If a path was provided and it does not exist.
    """
    def __init__(self, model):
        if isinstance(model, STRING_TYPES) or isinstance(model, Path):
            m_path = Path(model)
            if not Path(m_path).exists():
                raise ValueError('Model {} does not exist.'.format(m_path))
            model = cv2.CascadeClassifier(str(m_path))
        self._opencv_model = model

    def __call__(self, uint8_image, scale_factor=1.1, min_neighbours=5,
                 min_size=(30, 30), flags=None):
        r"""
        Perform a detection using the cached opencv detector.

        Parameters
        ----------
        uint8_image : `ndarray`
            A Greyscale (1 Channel) numpy array of uint8
        scale_factor : `float`, optional
            The amount to increase the sliding windows by over the second
            pass.
        min_neighbours : `int`, optional
            The minimum number of neighbours (close detections) before
            Non-Maximum suppression to be considered a detection. Use 0
            to return all detections.
        min_size : `tuple` of 2 ints
            The minimum object size in pixels that the detector will consider.
        flags : `int`
            The flags to be passed through to the detector.

        Returns
        ------
        bounding_boxes : `list` of `menpo.shape.PointDirectedGraph`
            The detected objects.
        """
        if flags is None:
            flags = _get_default_flags()
        rects = self._opencv_model.detectMultiScale(
            uint8_image, scaleFactor=scale_factor, minNeighbors=min_neighbours,
            minSize=min_size, flags=flags)
        return [pointgraph_from_rect(r) for r in rects]


class OpenCVDetector(object):
    r"""
    A generic opencv detector.

    Wraps an opencv object detector inside the menpodetect framework and
    provides a clean interface to expose the opencv arguments.
    """
    def __init__(self, model):
        self._detector = _opencv_detect(model)

    def __call__(self, image, image_diagonal=None, group_prefix='opencv',
                 scale_factor=1.1, min_neighbours=5,
                 min_size=(30, 30), flags=None):
        r"""
        Perform a detection using the cached opencv detector.

        The detections will also be attached to the image as landmarks.

        Parameters
        ----------
        image : `menpo.image.Image`
            A Menpo image to detect. The bounding boxes of the detected objects
            will be attached to this image.
        image_diagonal : `int`, optional
            The total size of the diagonal of the image that should be used for
            detection. This is useful for scaling images up and down for
            detection.
        group_prefix : `str`, optional
            The prefix string to be appended to each each landmark group that is
            stored on the image. Each detection will be stored as group_prefix_#
            where # is a count starting from 0.
        scale_factor : `float`, optional
            The amount to increase the sliding windows by over the second
            pass.
        min_neighbours : `int`, optional
            The minimum number of neighbours (close detections) before
            Non-Maximum suppression to be considered a detection. Use 0
            to return all detections.
        min_size : `tuple` of 2 ints
            The minimum object size in pixels that the detector will consider.
        flags : `int`, optional
            The flags to be passed through to the detector.

        Returns
        ------
        bounding_boxes : `list` of `menpo.shape.PointDirectedGraph`
            The detected objects.
        """
        if flags is None:
            flags = _get_default_flags()
        detect_partial = partial(self._detector, scale_factor=scale_factor,
                                 min_neighbours=min_neighbours,
                                 min_size=min_size, flags=flags)
        return detect(detect_partial, image, greyscale=True,
                      image_diagonal=image_diagonal, group_prefix=group_prefix)


def load_opencv_frontal_face_detector():
    r"""
    Load the opencv frontal face detector: haarcascade_frontalface_alt.xml

    Returns
    -------
    detector : OpenCVDetector
        The frontal face detector.
    """
    return OpenCVDetector(opencv_frontal_face_path)


def load_opencv_profile_face_detector():
    r"""
    Load the opencv profile face detector: haarcascade_profileface.xml

    Returns
    -------
    detector : OpenCVDetector
        The profile face detector.
    """
    return OpenCVDetector(opencv_profile_face_path)


def load_opencv_eye_detector():
    r"""
    Load the opencv eye detector: haarcascade_eye.xml

    Returns
    -------
    detector : OpenCVDetector
        The eye detector.
    """
    return OpenCVDetector(opencv_eye_path)




from __future__ import division
from functools import partial
from pathlib import Path

from menpo.base import MenpoMissingDependencyError

try:
    from cyffld2 import (load_model, detect_objects,
                         get_frontal_face_mixture_model)
except ImportError:
    raise MenpoMissingDependencyError('cyffld2')

from menpodetect.detect import detect
from menpodetect.compatibility import STRING_TYPES
from .conversion import pointgraph_from_rect, ensure_channel_axis


class _ffld2_detect(object):
    r"""
    A utility callable that allows the caching of an ffld2 detector.

    This callable is important for presenting the correct parameters to the
    user. It also marshalls the return type of the detector back to
    menpo.shape.PointDirectedGraph.

    Parameters
    ----------
    model : `Path` or `str` or `cyffld2.FFLDMixture`
        Either a path to an `cyffld2.FFLDMixture` or the detector itself.

    Raises
    ------
    ValueError
        If a path was provided and it does not exist.
    """
    def __init__(self, model):
        if isinstance(model, STRING_TYPES) or isinstance(model, Path):
            m_path = Path(model)
            if not Path(m_path).exists():
                raise ValueError('Model {} does not exist.'.format(m_path))
            model = load_model(str(m_path))
        self._ffld2_model = model

    def __call__(self, uint8_image, padding=6, interval=5, threshold=0.5,
                 overlap=0.3):
        r"""
        Perform a detection using the cached ffld2 detector.

        Parameters
        ----------
        uint8_image : `ndarray`
            A Greyscale or RGB image.
        padding : `int`, optional
            Amount of zero padding in HOG cells
        interval : `int`, optional
            Number of levels per octave in the HOG pyramid
        threshold : `double`
            Minimum detection threshold. Detections with a score less than this
            value are not returned. Values can be negative.
        overlap : `double`, optional
            Minimum overlap in in latent positive search and
            non-maxima suppression.
            As discussed in the Face Detection Without Bells and Whistles paper,
            a sensible value for overlap is 0.3

        Returns
        ------
        bounding_boxes : `list` of `menpo.shape.PointDirectedGraph`
            The detected objects.
        """
        # Add the channel to a greyscale image.
        uint8_image = ensure_channel_axis(uint8_image)
        rects = detect_objects(self._ffld2_model, uint8_image,
                               padding=padding, interval=interval,
                               threshold=threshold, overlap=overlap)
        return [pointgraph_from_rect(r) for r in rects]


class FFLD2Detector(object):
    r"""
    A generic ffld2 detector.

    Wraps an ffld2 object detector inside the menpodetect framework and
    provides a clean interface to expose the ffld2 arguments.
    """
    def __init__(self, model):
        self._detector = _ffld2_detect(model)

    def __call__(self, image, greyscale=True, image_diagonal=None,
                 group_prefix='ffld2', padding=6, interval=5, threshold=0.5,
                 overlap=0.3):
        r"""
        Perform a detection using the cached ffdl2 detector.

        The detections will also be attached to the image as landmarks.

        Parameters
        ----------
        image : `menpo.image.Image`
            A Menpo image to detect. The bounding boxes of the detected objects
            will be attached to this image.
        greyscale : `bool`, optional
            Whether to convert the image to greyscale or not.
        image_diagonal : `int`, optional
            The total size of the diagonal of the image that should be used for
            detection. This is useful for scaling images up and down for
            detection.
        group_prefix : `str`, optional
            The prefix string to be appended to each each landmark group that is
            stored on the image. Each detection will be stored as group_prefix_#
            where # is a count starting from 0.
        padding : `int`, optional
            Amount of zero padding in HOG cells
        interval : `int`, optional
            Number of levels per octave in the HOG pyramid
        threshold : `double`, optional
            Minimum detection threshold. Detections with a score less than this
            value are not returned. Values can be negative.
        overlap : `double`, optional
            Minimum overlap in in latent positive search and
            non-maxima suppression.
            As discussed in the Face Detection Without Bells and Whistles paper,
            a sensible value for overlap is 0.3

        Returns
        ------
        bounding_boxes : `list` of `menpo.shape.PointDirectedGraph`
            The detected objects.
        """
        detect_partial = partial(self._detector, padding=padding,
                                 interval=interval, threshold=threshold,
                                 overlap=overlap)
        return detect(detect_partial, image, greyscale=greyscale,
                      image_diagonal=image_diagonal, group_prefix=group_prefix)


def load_ffld2_frontal_face_detector():
    r"""
    Load the ffld2 frontal face detector. This detector is the DPM baseline
    provided from [1]_.

    Returns
    -------
    detector : FFLD2Detector
        The frontal face detector.

    References
    ----------
    .. [1] M. Mathias and R. Benenson and M. Pedersoli and L. Van Gool
       Face detection without bells and whistles
       ECCV 2014
    """
    return FFLD2Detector(get_frontal_face_mixture_model())



from mock import MagicMock
import numpy as np
from numpy.testing import assert_allclose

from menpo.shape import PointDirectedGraph
from menpodetect.detect import (detect, menpo_image_to_uint8)
import menpo.io as mio


takeo = mio.import_builtin_asset.takeo_ppm()
takeo_uint8 = mio.import_image(mio.data_path_to('takeo.ppm'), normalize=False)
fake_box = np.array([[0, 0], [1, 0], [1, 1], [0, 1]])
fake_detector = lambda x: ([PointDirectedGraph.init_from_edges(
    fake_box.copy(),
    np.array([[0, 1], [1, 2], [2, 3], [3, 0]]))])


def test_rescaling_image():
    takeo_copy = takeo.copy()
    ratio = 200.0 / takeo_copy.diagonal()
    pcs = detect(fake_detector, takeo_copy, image_diagonal=200)
    assert len(pcs) == 1
    assert takeo_copy.n_channels == 3
    assert takeo_copy.landmarks['object_0'].n_points == 4
    assert_allclose(takeo_copy.landmarks['object_0'].points,
                    fake_box * (1.0 / ratio), atol=10e-2)


def test_passing_uint8_image():
    takeo_copy = takeo_uint8.copy()
    pcs = detect(fake_detector, takeo_copy, greyscale=False)
    assert len(pcs) == 1
    assert takeo_copy.n_channels == 3
    assert takeo_copy.landmarks['object_0'].n_points == 4


def test_passing_uint8_image_greyscale():
    takeo_copy = takeo_uint8.copy()
    pcs = detect(fake_detector, takeo_copy, greyscale=True)
    assert len(pcs) == 1


def test_passing_uint8_rgb_image_greyscale_no_convert():
    fake_image = MagicMock()
    fake_image.n_channels = 3
    pcs = detect(fake_detector, fake_image, greyscale=True)
    assert len(pcs) == 1
    fake_image.as_greyscale.assert_called_once_with(mode='luminosity')


def test_passing_uint8_greyscale_image_greyscale_pass_through():
    fake_image = MagicMock()
    fake_image.n_channels = 1
    pcs = detect(fake_detector, fake_image, greyscale=True)
    assert len(pcs) == 1
    fake_image.as_greyscale.assert_not_called()


def test_image_to_uint8():
    takeo_copy = takeo.copy()
    np_im = menpo_image_to_uint8(takeo_copy)
    shi = takeo_copy.pixels.shape
    shnp = np_im.shape
    assert np_im.dtype == np.uint8
    assert (shi[0] == shnp[2] and shi[1] == shnp[0] and shi[2] == shnp[1])


def test_image_to_uint8_greyscale():
    takeo_copy = takeo.as_greyscale()
    np_im = menpo_image_to_uint8(takeo_copy)
    shi = takeo_copy.pixels.shape
    shnp = np_im.shape
    assert np_im.ndim == 2
    assert np_im.dtype == np.uint8
    assert (shi[1] == shnp[0] and shi[2] == shnp[1])


def test_image_to_uint8_channels_at_front():
    takeo_copy = takeo.copy()
    np_im = menpo_image_to_uint8(takeo_copy, channels_at_back=False)
    shi = takeo_copy.pixels.shape
    shnp = np_im.shape
    assert np_im.dtype == np.uint8
    assert (shi[0] == shnp[0] and shi[1] == shnp[1] and shi[2] == shnp[2])


def test_image_to_uint8_greyscale_channels_at_front():
    takeo_copy = takeo.as_greyscale()
    np_im = menpo_image_to_uint8(takeo_copy, channels_at_back=False)
    shi = takeo_copy.pixels.shape
    shnp = np_im.shape
    assert np_im.ndim == 2
    assert np_im.dtype == np.uint8
    assert (shi[1] == shnp[0] and shi[2] == shnp[1])



import sys

if sys.version_info.major == 3:
    from nose.plugins.skip import SkipTest
    raise SkipTest('OpenCV is incompatible with Python 3 - '
                   'skipping OpenCV tests')

from menpodetect.opencv import (load_opencv_frontal_face_detector,
                                load_opencv_eye_detector)
import menpo.io as mio

takeo = mio.import_builtin_asset.takeo_ppm()


def test_frontal_face_detector():
    takeo_copy = takeo.copy()
    opencv_detector = load_opencv_frontal_face_detector()
    pcs = opencv_detector(takeo_copy)
    assert len(pcs) == 1
    assert takeo_copy.n_channels == 3
    assert takeo_copy.landmarks['opencv_0'].n_points == 4


def test_frontal_face_detector_min_neighbors():
    takeo_copy = takeo.copy()
    opencv_detector = load_opencv_frontal_face_detector()
    pcs = opencv_detector(takeo_copy, min_neighbours=100)
    assert len(pcs) == 0
    assert takeo_copy.n_channels == 3


def test_eye_detector():
    takeo_copy = takeo.copy()
    opencv_detector = load_opencv_eye_detector()
    pcs = opencv_detector(takeo_copy, min_size=(5, 5), min_neighbours=0)
    assert len(pcs) > 0
    assert takeo_copy.n_channels == 3
    # This is because appyveyor and travis (automated testing) return
    # a different number of detections
    first_l = list(takeo_copy.landmarks.items_matching('opencv_*'))[0][1]
    assert first_l.n_points == 4



from __future__ import division
import numpy as np
from menpo.transform import UniformScale


def _greyscale(image):
    r"""
    Convert image to greyscale if needed. If the image has more than 3 channels,
    then the average greyscale is taken. A copy of the image as greyscale is
    returned (single channel).

    Parameters
    ----------
    image : `menpo.image.Image`
        The image to convert.

    Returns
    -------
    image : `menpo.image.Image`
        A greyscale version of the image.
    """
    if image.n_channels != 1:
        if image.n_channels == 3:
            # Use luminosity for RGB images
            image = image.as_greyscale(mode='luminosity')
        else:
            # Fall back to the average of the channels for other kinds
            # of images
            image = image.as_greyscale(mode='average')
    return image


def menpo_image_to_uint8(image, channels_at_back=True):
    r"""
    Return the given image as a uint8 array. This is a copy of the image.

    Parameters
    ----------
    image : `menpo.image.Image`
        The image to convert. If already uint8, only the channels will be
        rolled to the last axis.
    channels_at_back : `bool`, optional
        If ``True``, the image channels are placed onto the last axis (the back)
        as is common in many imaging packages. This is contrary to the Menpo
        default where channels are the first axis (at the front).

    Returns
    -------
    uint8_image : `ndarray`
        `uint8` Numpy array, channels as the back (last) axis if
        ``channels_at_back == True``.
    """
    if channels_at_back:
        uint8_im = image.pixels_with_channels_at_back(out_dtype=np.uint8)
        # Handle the dead axis on greyscale images
        if uint8_im.ndim == 3 and uint8_im.shape[-1] == 1:
            uint8_im = uint8_im[..., 0]
    else:
        from menpo.image.base import denormalize_pixels_range
        uint8_im = denormalize_pixels_range(image.pixels, np.uint8)
        # Handle the dead axis on greyscale images
        if uint8_im.ndim == 3 and uint8_im.shape[0] == 1:
            uint8_im = uint8_im[0]
    return uint8_im


def detect(detector_callable, image, greyscale=True,
           image_diagonal=None, group_prefix='object', channels_at_back=True):
    r"""
    Apply the general detection framework.

    This involves converting the image to greyscale if necessary, rescaling
    the image to a given diagonal, performing the detection, and attaching
    the scaled landmarks back onto the original image.

    uint8 images cannot be converted to greyscale by this framework, so must
    already be greyscale or ``greyscale=False``.

    Parameters
    ----------
    detector_callable : `callable` or `function`
        A callable object that will perform detection given a single parameter,
        a `uint8` numpy array with either no channels, or channels as the
        *last* axis.
    image : `menpo.image.Image`
        A Menpo image to detect. The bounding boxes of the detected objects
        will be attached to this image.
    greyscale : `bool`, optional
        Convert the image to greyscale or not.
    image_diagonal : `int`, optional
        The total size of the diagonal of the image that should be used for
        detection. This is useful for scaling images up and down for detection.
    group_prefix : `str`, optional
        The prefix string to be appended to each each landmark group that is
        stored on the image. Each detection will be stored as group_prefix_#
        where # is a count starting from 0.
    channels_at_back : `bool`, optional
        If ``True``, the image channels are placed onto the last axis (the back)
        as is common in many imaging packages. This is contrary to the Menpo
        default where channels are the first axis (at the front).

    Returns
    -------
    bounding_boxes : `list` of `menpo.shape.PointDirectedGraph`
        A list of bounding boxes representing the detections found.
    """
    d_image = image

    if greyscale:
        d_image = _greyscale(d_image)

    if image_diagonal is not None:
        scale_factor = image_diagonal / image.diagonal()
        d_image = d_image.rescale(scale_factor)

    pcs = detector_callable(menpo_image_to_uint8(
        d_image, channels_at_back=channels_at_back))

    if image_diagonal is not None:
        s = UniformScale(1 / scale_factor, n_dims=2)
        pcs = [s.apply(pc) for pc in pcs]

    padding_magnitude = len(str(len(pcs)))
    for i, pc in enumerate(pcs):
        key = '{prefix}_{num:0{mag}d}'.format(mag=padding_magnitude,
                                              prefix=group_prefix, num=i)
        image.landmarks[key] = pc
    return pcs



from menpo.shape import bounding_box


def pointgraph_from_rect(rect):
    r"""
    Convert an ffld2 detection to a menpo.shape.PointDirectedGraph.
    This enforces a particular point ordering.

    Parameters
    ----------
    rect : `tuple`
        The bounding box to convert. Result of calling an opencv detection.

    Returns
    -------
    bounding_box : `menpo.shape.PointDirectedGraph`
        A menpo PointDirectedGraph giving the bounding box.
    """
    return bounding_box((rect.y, rect.x),
                        (rect.y + rect.height, rect.x + rect.width))


def ensure_channel_axis(image_pixels):
    # In the case of greyscale images, we need to manually add the channel
    # back on for the Cython code.
    if image_pixels.ndim == 2:
        image_pixels = image_pixels[..., None]
    return image_pixels



from __future__ import division
from functools import partial
from pathlib import Path
import numpy as np

from menpo.base import MenpoMissingDependencyError

try:
    from cypico import detect_objects, detect_frontal_faces
except ImportError:
    raise MenpoMissingDependencyError('cypico')

from menpodetect.detect import detect
from menpodetect.compatibility import STRING_TYPES
from .conversion import pointgraph_from_circle


class _pico_detect(object):
    r"""
    A utility callable that allows the caching of a pico detector.

    This callable is important for presenting the correct parameters to the
    user. It also marshalls the return type of the detector back to
    menpo.shape.PointDirectedGraph.

    Parameters
    ----------
    model : `pico detector`
        At the moment loading new pico detectors is not possible. Unless you
        have managed to load a pico detector as a 1D uint8 array.

    Raises
    ------
    ValueError
        If a path was provided.
    """
    def __init__(self, model):
        if isinstance(model, STRING_TYPES) or isinstance(model, Path):
            raise ValueError('Loading Pico trained models from disk '
                             'is not currently supported.')
        self._pico_model = model

    def __call__(self, uint8_image, max_detections=100, orientations=0.0,
                 degrees=True, scale_factor=1.2, stride_factor=0.1,
                 min_size=100, confidence_cutoff=3.0, axis_aligned_bb=True):
        r"""
        Perform a detection using the cached pico detector.

        Parameters
        ----------
        uint8_image : `ndarray`
            A Greyscale (1 Channel) numpy array of uint8
        max_detections : `int`, optional
            The maximum number of detections to return.
        orientations : `list` of `float`s or `float`, optional
            The orientations of the cascades to use. ``0.0`` will perform an
            axis aligned detection. Values greater than ``0.0`` will perform
            detections of the cascade rotated counterclockwise around a unit
            circle.
            If a list is passed, each item should be an orientation in
            radians around the unit circle, with ``0.0`` being axis aligned.
        scale_factor : `float`, optional
            The ratio to increase the cascade window at every iteration. Must
            be greater than 1.0
        stride_factor : `float`, optional
            The ratio to decrease the window step by at every iteration. Must be
            less than 1.0, optional
        min_size : `float`, optional
            The minimum size in pixels (diameter of the detection circle) that a
            face can be. This is the starting cascade window size.
        confidence_cutoff : `float`, optional
            The confidence value to trim the detections with. Any detections
            with confidence less than the cutoff will be discarded.
        axis_aligned_bb : `bool`, optional
            If ``True``, the returned detections will be axis aligned,
            regardless of which orientation they were detected at.
            If ``False``, the returned bounding box will be rotated by the
            orientation detected.

        Returns
        ------
        bounding_boxes : `list` of `menpo.shape.PointDirectedGraph`
            The detected objects.
        """
        fittings = detect_objects(
            uint8_image, self._pico_model, max_detections=max_detections,
            orientations=orientations, scale_factor=scale_factor,
            stride_factor=stride_factor, min_size=min_size,
            confidence_cutoff=confidence_cutoff,
            axis_aligned_bb=axis_aligned_bb)
        return [pointgraph_from_circle(f) for f in fittings]


class _pico_face_detect(_pico_detect):
    def __call__(self, uint8_image, max_detections=100, orientations=0.0,
                 scale_factor=1.2, stride_factor=0.1,
                 min_size=100, confidence_cutoff=3.0, axis_aligned_bb=True):
        r"""
        Perform a detection using the frontal face pico detector.

        The detections will also be attached to the image as landmarks.

        Parameters
        ----------
        image : `menpo.image.Image`
            A Menpo image to detect. The bounding boxes of the detected faces
            will be attached to this image.
        image_diagonal : `int`, optional
            The total size of the diagonal of the image that should be used for
            detection. This is useful for scaling images up and down for
            detection.
        group_prefix : `str`, optional
            The prefix string to be appended to each each landmark group that is
            stored on the image. Each detection will be stored as group_prefix_#
            where # is a count starting from 0.
        max_detections : `int`, optional
            The maximum number of detections to return.
        orientations : list of `float`s or `float`, optional
            The orientations of the cascades to use. ``0.0`` will perform an
            axis aligned detection. Values greater than ``0.0`` will perform
            detections of the cascade rotated counterclockwise around a unit
            circle.
            If a list is passed, each item should be an orientation in
            radians around the unit circle, with ``0.0`` being axis aligned.
        scale_factor : `float`, optional
            The ratio to increase the cascade window at every iteration. Must
            be greater than 1.0
        stride_factor : `float`, optional
            The ratio to decrease the window step by at every iteration. Must be
            less than 1.0, optional
        min_size : `float`, optional
            The minimum size in pixels (diameter of the detection circle) that a
            face can be. This is the starting cascade window size.
        confidence_cutoff : `float`, optional
            The confidence value to trim the detections with. Any detections
            with confidence less than the cutoff will be discarded.
        axis_aligned_bb : `bool`, optional
            If ``True``, the returned detections will be axis aligned,
            regardless of which orientation they were detected at.
            If ``False``, the returned bounding box will be rotated by the
            orientation detected.

        Returns
        ------
        bounding_boxes : `list` of `menpo.shape.PointDirectedGraph`
            The detected faces.
        """
        fittings = detect_frontal_faces(
            uint8_image, max_detections=max_detections,
            orientations=orientations, scale_factor=scale_factor,
            stride_factor=stride_factor, min_size=min_size,
            confidence_cutoff=confidence_cutoff)
        return [pointgraph_from_circle(f, axis_aligned_bb=axis_aligned_bb)
                for f in fittings]


# This is a slightly strange model because at the moment cypico is not
# really able to load arbitrary pico models. This is due to the fact that
# pico models are just saved down as raw data and loaded into a char* array
# then case into the members of the C-struct they actually represent. To make
# this cleaner, the C-struct would need to be created in cypico so we could
# properly load and save pico models
class PicoDetector(object):
    r"""
    A generic pico detector.

    Wraps a pico object detector inside the menpodetect framework and
    provides a clean interface to expose the pico arguments.

    At the moment this isn't particularly useful as loading Pico models is
    complex.
    """
    def __init__(self, model, detector=_pico_detect):
        self._detector = detector(model)

    def __call__(self, image, image_diagonal=None, group_prefix='pico',
                 max_detections=100, orientations=0.0, degrees=True,
                 scale_factor=1.2, stride_factor=0.1, min_size=100,
                 confidence_cutoff=3.0, axis_aligned_bb=True):
        r"""
        Perform a detection using the cached pico detector.

        The detections will also be attached to the image as landmarks.

        Parameters
        ----------
        image : `menpo.image.Image`
            A Menpo image to detect. The bounding boxes of the detected objects
            will be attached to this image.
        image_diagonal : `int`, optional
            The total size of the diagonal of the image that should be used for
            detection. This is useful for scaling images up and down for
            detection.
        group_prefix : `str`, optional
            The prefix string to be appended to each each landmark group that is
            stored on the image. Each detection will be stored as group_prefix_#
            where # is a count starting from 0.
        max_detections : `int`, optional
            The maximum number of detections to return.
        orientations : list of `float`s or `float`, optional
            The orientations of the cascades to use. ``0.0`` will perform an
            axis aligned detection. Values greater than ``0.0`` will perform
            detections of the cascade rotated counterclockwise around a unit
            circle.
            If a list is passed, each item should be an orientation in
            either radians or degrees around the unit circle, with ``0.0``
            being axis aligned.
        degrees : `bool`, optional
            If ``True``, the ``orientations`` parameter is treated as
            rotations counterclockwise in degrees rather than radians.
        scale_factor : `float`, optional
            The ratio to increase the cascade window at every iteration. Must
            be greater than 1.0
        stride_factor : `float`, optional
            The ratio to decrease the window step by at every iteration. Must be
            less than 1.0, optional
        min_size : `float`, optional
            The minimum size in pixels (diameter of the detection circle) that a
            face can be. This is the starting cascade window size.
        confidence_cutoff : `float`, optional
            The confidence value to trim the detections with. Any detections
            with confidence less than the cutoff will be discarded.
        axis_aligned_bb : `bool`, optional
            If ``True``, the returned detections will be axis aligned,
            regardless of which orientation they were detected at.
            If ``False``, the returned bounding box will be rotated by the
            orientation detected.

        Returns
        ------
        bounding_boxes : `list` of `menpo.shape.PointDirectedGraph`
            The detected objects.
        """
        if degrees:
            # Cypico expects Radians
            orientations = np.deg2rad(orientations)
        detect_partial = partial(self._detector, max_detections=max_detections,
                                 orientations=orientations,
                                 scale_factor=scale_factor,
                                 stride_factor=stride_factor, min_size=min_size,
                                 confidence_cutoff=confidence_cutoff,
                                 axis_aligned_bb=axis_aligned_bb)
        return detect(detect_partial, image, greyscale=True,
                      image_diagonal=image_diagonal, group_prefix=group_prefix)


def load_pico_frontal_face_detector():
    r"""
    Load the pico frontal face detector.

    Returns
    -------
    detector : `PicoDetector`
        The frontal face detector.
    """
    return PicoDetector(None, detector=_pico_face_detect)



import bob.ip.facedetect
from menpo.shape import bounding_box


def bb_to_pointgraph(bb):
    r"""
    Convert a `bob.ip.facedetect.BoundingBox` to a
    `menpo.shape.PointDirectedGraph`.
    This enforces a particular point ordering.

    Parameters
    ----------
    bb : `bob.ip.facedetect.BoundingBox`
        The bounding box to convert.

    Returns
    -------
    bounding_box : `menpo.shape.PointDirectedGraph`
        A menpo PointDirectedGraph giving the bounding box.
    """
    return bounding_box(bb.topleft_f, bb.bottomright_f)


def pointgraph_to_bb(pg):
    r"""
    Convert a `menpo.shape.PointCloud` to a `bob.ip.facedetect.BoundingBox`.

    Parameters
    ----------
    pg : `menpo.shape.PointDirectedGraph`
        The Menpo PointDirectedGraph to convert into a rect. No check is done
        to see if the PointDirectedGraph actually is a rectangle.

    Returns
    -------
    bounding_box : `bob.ip.facedetect.BoundingBox`
        A bob BoundingBox.
    """
    return bob.ip.facedetect.BoundingBox(pg.bounds()[0], pg.range())



from menpo.base import MenpoMissingDependencyError


try:
    from .detect import load_bob_frontal_face_detector, BobDetector
except MenpoMissingDependencyError:
    pass

# Remove from scope
del MenpoMissingDependencyError



import os


def src_dir_path():
    r"""The path to the top of the MenpoDetect Python package.

    Useful for locating where the models folder is stored.

    Returns
    -------
    path : `str`
        The full path to the top of the MenpoDetect package
    """
    return os.path.split(os.path.abspath(__file__))[0]


def models_dir_path():
    r"""The path to the models directory of the MenpoDetect Python package.

    Returns
    -------
    path : `str`
        The full path to the models directory of the MenpoDetect package
    """
    return os.path.join(src_dir_path(), 'models')



import dlib
from menpo.shape import bounding_box


def rect_to_pointgraph(rect):
    r"""
    Convert a dlib.rect to a menpo.shape.PointDirectedGraph.
    This enforces a particular point ordering.

    Parameters
    ----------
    rect : `dlib.rect`
        The bounding box to convert.

    Returns
    -------
    bounding_box : `menpo.shape.PointDirectedGraph`
        A menpo PointDirectedGraph giving the bounding box.
    """
    return bounding_box((rect.top(), rect.left()),
                        (rect.bottom(), rect.right()))


def pointgraph_to_rect(pg):
    r"""
    Convert a `menpo.shape.PointCloud` to a `dlib.rect`.

    Parameters
    ----------
    pg : `menpo.shape.PointDirectedGraph`
        The Menpo PointDirectedGraph to convert into a rect. No check is done
        to see if the PointDirectedGraph actually is a rectangle.

    Returns
    -------
    bounding_rect : `dlib.rect`
        A dlib Rectangle.
    """
    min_p, max_p = pg.bounds()
    return dlib.rectangle(left=int(min_p[1]), top=int(min_p[0]),
                          right=int(max_p[1]), bottom=int(max_p[0]))



import unittest
try:
    from menpodetect.bob import load_bob_frontal_face_detector
    MISSING_BOB = False
except ImportError:
    MISSING_BOB = True

import menpo.io as mio

takeo = mio.import_builtin_asset.takeo_ppm()


@unittest.skipIf(MISSING_BOB, "requires bob.ip.facedetect")
def test_frontal_face_detector():
    takeo_copy = takeo.copy()
    bob_detector = load_bob_frontal_face_detector()
    pcs = bob_detector(takeo_copy)
    assert len(pcs) == 2
    assert takeo_copy.n_channels == 3
    assert takeo_copy.landmarks['bob_0'].n_points == 4


@unittest.skipIf(MISSING_BOB, "requires bob.ip.facedetect")
def test_frontal_face_detector_no_result():
    takeo_copy = takeo.copy()
    bob_detector = load_bob_frontal_face_detector()
    pcs = bob_detector(takeo_copy, threshold=float('inf'))
    assert len(pcs) == 0


@unittest.skipIf(MISSING_BOB, "requires bob.ip.facedetect")
def test_frontal_face_detector_rgb():
    takeo_copy = takeo.copy()
    bob_detector = load_bob_frontal_face_detector()
    pcs = bob_detector(takeo_copy, greyscale=False)
    assert len(pcs) == 2
    assert takeo_copy.n_channels == 3
    assert takeo_copy.landmarks['bob_0'].n_points == 4


@unittest.skipIf(MISSING_BOB, "requires bob.ip.facedetect")
def test_frontal_face_detector_threshold():
    takeo_copy = takeo.copy()
    bob_detector = load_bob_frontal_face_detector()
    pcs = bob_detector(takeo_copy, threshold=30)
    assert len(pcs) == 1
    assert takeo_copy.n_channels == 3
    assert takeo_copy.landmarks['bob_0'].n_points == 4


@unittest.skipIf(MISSING_BOB, "requires bob.ip.facedetect")
def test_frontal_face_detector_minimum_overlap():
    takeo_copy = takeo.copy()
    bob_detector = load_bob_frontal_face_detector()
    pcs = bob_detector(takeo_copy, minimum_overlap=0.5)
    assert len(pcs) == 4
    assert takeo_copy.n_channels == 3
    assert takeo_copy.landmarks['bob_0'].n_points == 4



from menpo.shape import bounding_box
from menpo.transform import rotate_ccw_about_centre


def pointgraph_from_circle(fitting, axis_aligned_bb=True):
    r"""
    Convert a Pico detection to a menpo.shape.PointDirectedGraph.
    This enforces a particular point ordering. The Pico detections are
    circles with a given diameter. Here we convert them to the tightest
    possible bounding box around the circle. No orientation is currently
    provided.

    Parameters
    ----------
    fitting : `cypico.PicoDetection`
        The Pico detection to convert. Result of calling a pico detection.
        A namedtuple with a diameter and a centre.

    Returns
    -------
    bounding_box : `menpo.shape.PointDirectedGraph`
        A menpo PointDirectedGraph giving the bounding box.
    """
    diameter = fitting.diameter
    radius = diameter / 2.0
    y, x = fitting.center
    y -= radius
    x -= radius
    bb = bounding_box((y, x), (y + diameter, x + diameter))
    if not axis_aligned_bb:
        t = rotate_ccw_about_centre(bb, fitting.orientation, degrees=False)
        bb = t.apply(bb)
    return bb



from __future__ import division
from functools import partial
from pathlib import Path

from menpo.base import MenpoMissingDependencyError

try:
    import bob.ip.facedetect
    from bob.ip.facedetect.detector.cascade import Cascade
except ImportError:
    raise MenpoMissingDependencyError('bob.ip.facedetect')

from menpodetect.detect import detect
from menpodetect.compatibility import STRING_TYPES
from .conversion import bb_to_pointgraph


class _bob_detect(object):
    r"""
    A utility callable that allows the caching of a bob detector.

    This callable is important for presenting the correct parameters to the
    user. It also marshalls the return type of the detector back to
    `menpo.shape.PointDirectedGraph`.

    Parameters
    ----------
    model : `Path` or `str` or `bob.ip.facedetect.detector.cascade.Cascade`
        Either a path to a `bob.ip.facedetect.detector.cascade.Cascade` or a
        `bob.ip.facedetect.detector.cascade.Cascade` itself.

    Raises
    ------
    ValueError
        If a path was provided and it does not exist.
    """
    def __init__(self, model):
        if isinstance(model, STRING_TYPES) or isinstance(model, Path):
            m_path = Path(model)
            if not Path(m_path).exists():
                raise ValueError('Model {} does not exist.'.format(m_path))
            model = Cascade(cascade_file=str(m_path))
        self._bob_model = model

    def __call__(self, uint8_image, threshold=20, minimum_overlap=0.2):
        r"""
        Perform a detection using the cached bob detector.

        Parameters
        ----------
        uint8_image : `ndarray`
            An RGB (3 Channels) or Greyscale (1 Channel) numpy array of uint8
            with **channels at the front**.
        threshold : `float`, optional
            The threshold of the quality of detected faces.
            Detections with a quality lower than this value will not be
            considered. Higher thresholds will not detect all faces, while lower
            thresholds will generate false detections.
        ``minimum_overlap`` : `float` ``[0, 1]``
            Computes the best detection using the given minimum overlap.

        Returns
        ------
        bounding_boxes : `list` of `menpo.shape.PointDirectedGraph`
            The detected objects.
        """
        result = bob.ip.facedetect.detect_all_faces(
            uint8_image, cascade=self._bob_model, threshold=threshold,
            minimum_overlap=minimum_overlap)
        if result is None:
            bbs = []
        else:
            bbs, confidences = result
        return [bb_to_pointgraph(b) for b in bbs]


class BobDetector(object):
    r"""
    A Bob cascade detector.

    Wraps a bob cascade detector inside the menpodetect framework and provides
    a clean interface to expose the bob arguments.
    """
    def __init__(self, model):
        self._detector = _bob_detect(model)

    def __call__(self, image, greyscale=False, image_diagonal=None,
                 group_prefix='bob', threshold=20, minimum_overlap=0.2):
        r"""
        Perform a detection using the cached bob detector.

        The detections will also be attached to the image as landmarks.

        Parameters
        ----------
        image : `menpo.image.Image`
            A Menpo image to detect. The bounding boxes of the detected objects
            will be attached to this image.
        greyscale : `bool`, optional
            Convert the image to greyscale or not.
        image_diagonal : `int`, optional
            The total size of the diagonal of the image that should be used for
            detection. This is useful for scaling images up and down for
            detection.
        group_prefix : `str`, optional
            The prefix string to be appended to each each landmark group that is
            stored on the image. Each detection will be stored as group_prefix_#
            where # is a count starting from 0.
        threshold : `float`, optional
            The threshold of the quality of detected faces.
            Detections with a quality lower than this value will not be
            considered. Higher thresholds will not detect all faces, while lower
            thresholds will generate false detections.
        ``minimum_overlap`` : `float` ``[0, 1]``
            Computes the best detection using the given minimum overlap.

        Returns
        ------
        bounding_boxes : `list` of `menpo.shape.PointDirectedGraph`
            The detected objects.
        """
        detect_partial = partial(self._detector, threshold=threshold,
                                 minimum_overlap=minimum_overlap)
        return detect(detect_partial, image, greyscale=greyscale,
                      image_diagonal=image_diagonal, group_prefix=group_prefix,
                      channels_at_back=False)


def load_bob_frontal_face_detector():
    r"""
    Load the bob frontal face detector.

    Returns
    -------
    detector : `BobDetector`
        The frontal face detector.
    """
    return BobDetector(bob.ip.facedetect.default_cascade())



from menpodetect.dlib import load_dlib_frontal_face_detector
import menpo.io as mio

takeo = mio.import_builtin_asset.takeo_ppm()


def test_frontal_face_detector():
    takeo_copy = takeo.copy()
    dlib_detector = load_dlib_frontal_face_detector()
    pcs = dlib_detector(takeo_copy)
    assert len(pcs) == 1
    assert takeo_copy.n_channels == 3
    assert takeo_copy.landmarks['dlib_0'].n_points == 4


def test_frontal_face_detector_rgb():
    takeo_copy = takeo.copy()
    assert takeo_copy.n_channels == 3
    dlib_detector = load_dlib_frontal_face_detector()
    pcs = dlib_detector(takeo_copy, greyscale=False)
    assert len(pcs) == 1
    assert takeo_copy.n_channels == 3
    assert takeo_copy.landmarks['dlib_0'].n_points == 4


def test_frontal_face_detector_upscales():
    takeo_copy = takeo.copy()
    dlib_detector = load_dlib_frontal_face_detector()
    pcs = dlib_detector(takeo_copy, n_upscales=1)
    assert len(pcs) == 1
    assert takeo_copy.n_channels == 3
    assert takeo_copy.landmarks['dlib_0'].n_points == 4



from menpo.base import MenpoMissingDependencyError


try:
    from .detect import load_dlib_frontal_face_detector, DlibDetector
    from .train import train_dlib_detector
except MenpoMissingDependencyError:
    pass

# Remove from scope
del MenpoMissingDependencyError



from menpo.base import MenpoMissingDependencyError


try:
    from .detect import (load_opencv_frontal_face_detector,
                         load_opencv_profile_face_detector,
                         load_opencv_eye_detector, OpenCVDetector)
except MenpoMissingDependencyError:
    pass

# Remove from scope
del MenpoMissingDependencyError




import numpy as np

from menpo.base import MenpoMissingDependencyError

try:
    from cyffld2 import train_model
except ImportError:
    raise MenpoMissingDependencyError('cyffld2')

from menpodetect.detect import menpo_image_to_uint8
from .conversion import ensure_channel_axis


def train_ffld2_detector(positive_images, negative_images, n_components=3,
                         pad_x=6, pad_y=6, interval=5, n_relabel=8,
                         n_datamine=10, max_negatives=24000, C=0.002, J=2.0,
                         overlap=0.5):
    r"""
    Train a DPM using the FFLD2 framework. This is a fairly slow process to
    expect to wait for a while. FFLD2 prints out information at each iteration
    but this will not appear in an IPython notebook, so it is best to run
    this kind of training from the command line.

    This method requires an explicit set of negative images to learn the
    classifier with. The non person images from Pascal VOC 2007 are a good
    example of negative images to train with.

    Parameters
    ----------
    positive_images : `list` of `menpo.image.Image`
        The set of images to learn the detector from. Must have landmarks
        attached to **every** image, a bounding box will be extracted for each
        landmark group.
    negative_images : `list` of `menpo.image.Image`
        The set of images to learn the negative samples of the detector with.
        **No** landmarks need to be attached.
    n_components : `int`
        Number of mixture components (without symmetry).
    pad_x : `int`
        Amount of zero padding in HOG cells (x-direction).
    pad_y : `int`
        Amount of zero padding in HOG cells (y-direction).
    interval : `int`
        Number of levels per octave in the HOG pyramid.
    n_relabel : `int`
        Maximum number of training iterations.
    n_datamine : `int`
        Maximum number of data-mining iterations within each training iteration.
    max_negatives : `int`
        Maximum number of negative images to consider, can be useful for
        reducing training time.
    C : `double`
        SVM regularization constant.
    J : `double`
        SVM positive regularization constant boost.
    overlap : `double`
        Minimum overlap in in latent positive search and non-maxima suppression.

    Returns
    -------
    model : `FFLDMixture`
        The newly trained model.
    """
    positive_image_arrays = []
    negative_image_arrays = []
    positive_bbox_arrays = []

    for image in positive_images:
        image_pixels = menpo_image_to_uint8(image)
        image_pixels = ensure_channel_axis(image_pixels)
        positive_image_arrays.append(image_pixels)
        im_bounding_boxes = []
        for lmark in image.landmarks.values():
            bb = lmark.bounding_box()
            height, width = bb.range()
            min_p, max_p = bb.bounds()
            im_bounding_boxes.append(np.array([min_p[1], min_p[0],
                                               width, height]))
        positive_bbox_arrays.append(im_bounding_boxes)

    for image in negative_images:
        image_pixels = menpo_image_to_uint8(image)
        image_pixels = ensure_channel_axis(image_pixels)
        negative_image_arrays.append(image_pixels)

    return train_model(positive_image_arrays, positive_bbox_arrays,
                       negative_image_arrays, n_components=n_components,
                       pad_x=pad_x, pad_y=pad_y, interval=interval,
                       n_relabel=n_relabel, n_datamine=n_datamine,
                       max_negatives=max_negatives, C=C, J=J, overlap=overlap)



from __future__ import division
from functools import partial
from pathlib import Path

from menpo.base import MenpoMissingDependencyError

try:
    import dlib
except ImportError:
    raise MenpoMissingDependencyError('dlib')

from menpodetect.detect import detect
from menpodetect.compatibility import STRING_TYPES
from .conversion import rect_to_pointgraph


class _dlib_detect(object):
    r"""
    A utility callable that allows the caching of a dlib detector.

    This callable is important for presenting the correct parameters to the
    user. It also marshalls the return type of the detector back to
    `menpo.shape.PointDirectedGraph`.

    Parameters
    ----------
    model : `Path` or `str` or `dlib.simple_object_detector`
        Either a path to a `dlib.simple_object_detector` or a
        `dlib.fhog_object_detector` or the detector itself.

    Raises
    ------
    ValueError
        If a path was provided and it does not exist.
    """
    def __init__(self, model):
        if isinstance(model, STRING_TYPES) or isinstance(model, Path):
            m_path = Path(model)
            if not Path(m_path).exists():
                raise ValueError('Model {} does not exist.'.format(m_path))
            # There are two different kinds of object detector, the
            # simple_object_detector and the fhog_object_detector, but we
            # can't tell which is which from the file name. Therefore, try one
            # and then the other. Unfortunately, it throws a runtime error,
            # which we have to catch.
            try:
                model = dlib.simple_object_detector(str(m_path))
            except RuntimeError:
                model = dlib.fhog_object_detector(str(m_path))
        self._dlib_model = model

    def __call__(self, uint8_image, n_upscales=0):
        r"""
        Perform a detection using the cached dlib detector.

        Parameters
        ----------
        uint8_image : `ndarray`
            An RGB (3 Channels) or Greyscale (1 Channel) numpy array of uint8
        n_upscales : `int`, optional
            Number of times to upscale the image when performing the detection,
            may increase the chances of detecting smaller objects.

        Returns
        ------
        bounding_boxes : `list` of `menpo.shape.PointDirectedGraph`
            The detected objects.
        """
        # Dlib doesn't handle the dead last axis
        if uint8_image.shape[-1] == 1:
            uint8_image = uint8_image[..., 0]
        rects = self._dlib_model(uint8_image, n_upscales)
        return [rect_to_pointgraph(r) for r in rects]


class DlibDetector(object):
    r"""
    A generic dlib detector.

    Wraps a dlib object detector inside the menpodetect framework and provides
    a clean interface to expose the dlib arguments.
    """
    def __init__(self, model):
        self._detector = _dlib_detect(model)

    def __call__(self, image, greyscale=False, image_diagonal=None,
                 group_prefix='dlib', n_upscales=0):
        r"""
        Perform a detection using the cached dlib detector.

        The detections will also be attached to the image as landmarks.

        Parameters
        ----------
        image : `menpo.image.Image`
            A Menpo image to detect. The bounding boxes of the detected objects
            will be attached to this image.
        greyscale : `bool`, optional
            Convert the image to greyscale or not.
        image_diagonal : `int`, optional
            The total size of the diagonal of the image that should be used for
            detection. This is useful for scaling images up and down for
            detection.
        group_prefix : `str`, optional
            The prefix string to be appended to each each landmark group that is
            stored on the image. Each detection will be stored as group_prefix_#
            where # is a count starting from 0.
        n_upscales : `int`, optional
            Number of times to upscale the image when performing the detection,
            may increase the chances of detecting smaller objects.

        Returns
        ------
        bounding_boxes : `list` of `menpo.shape.PointDirectedGraph`
            The detected objects.
        """
        detect_partial = partial(self._detector, n_upscales=n_upscales)
        return detect(detect_partial, image, greyscale=greyscale,
                      image_diagonal=image_diagonal, group_prefix=group_prefix)


def load_dlib_frontal_face_detector():
    r"""
    Load the dlib frontal face detector.

    Returns
    -------
    detector : `DlibDetector`
        The frontal face detector.
    """
    return DlibDetector(dlib.get_frontal_face_detector())



from setuptools import setup, find_packages
import versioneer


setup(name='menpodetect',
      version=versioneer.get_version(),
      cmdclass=versioneer.get_cmdclass(),
      description='Object detection for Menpo',
      author='The Menpo Development Team',
      author_email='james.booth08@imperial.ac.uk',
      packages=find_packages(),
      tests_require=['nose'],
      package_data={'menpodetect': ['models/opencv/*.xml']},
      install_requires=['menpo>=0.8,<0.9'])




from menpo.base import MenpoMissingDependencyError

try:
    import dlib
except ImportError:
    raise MenpoMissingDependencyError('dlib')

from menpodetect.detect import menpo_image_to_uint8
from .conversion import pointgraph_to_rect


def train_dlib_detector(images, epsilon=0.01, add_left_right_image_flips=False,
                        verbose_stdout=False, C=5, detection_window_size=6400,
                        num_threads=None):
    r"""
    Train a dlib detector with the given list of images.

    This is intended to easily train a list of menpo images that have their
    bounding boxes attached as landmarks. Each landmark group on the image
    will have a tight bounding box extracted from it and then dlib will
    train given these images.

    Parameters
    ----------
    images : `list` of `menpo.image.Image`
        The set of images to learn the detector from. Must have landmarks
        attached to **every** image, a bounding box will be extracted for each
        landmark group.
    epsilon : `float`, optional
        The stopping epsilon.  Smaller values make the trainer's solver more
        accurate but might take longer to train.
    add_left_right_image_flips : `bool`, optional
        If ``True``, assume the objects are left/right symmetric and add in
        left right flips of the training images.  This doubles the size of the
        training dataset.
    verbose_stdout : `bool`, optional
        If ``True``, will allow dlib to output its verbose messages. These
        will only be printed to the stdout, so will **not** appear in an IPython
        notebook.
    C : `int`, optional
        C is the usual SVM C regularization parameter.  Larger values of C will
        encourage the trainer to fit the data better but might lead to
        overfitting.
    detection_window_size : `int`, optional
        The number of pixels inside the sliding window used. The default
        parameter of ``6400 = 80 * 80`` window size.
    num_threads : `int` > 0 or ``None``
        How many threads to use for training. If ``None``, will query
        multiprocessing for the number of cores.

    Returns
    -------
    detector : `dlib.simple_object_detector`
        The trained detector. To save this detector, call save on the returned
        object and pass a string path.

    Examples
    --------
    Training a simple object detector from a list of menpo images and save it
    for later use:

    >>> images = list(mio.import_images('./images/path'))
    >>> in_memory_detector = train_dlib_detector(images, verbose_stdout=True)
    >>> in_memory_detector.save('in_memory_detector.svm')
    """
    rectangles = [[pointgraph_to_rect(lgroup.bounding_box())
                  for lgroup in im.landmarks.values()]
                  for im in images]
    image_pixels = [menpo_image_to_uint8(im) for im in images]

    if num_threads is None:
        import multiprocessing

        num_threads = multiprocessing.cpu_count()

    options = dlib.simple_object_detector_training_options()
    options.epsilon = epsilon
    options.add_left_right_image_flips = add_left_right_image_flips
    options.be_verbose = verbose_stdout
    options.C = C
    options.detection_window_size = detection_window_size
    options.num_threads = num_threads

    return dlib.train_simple_object_detector(image_pixels, rectangles, options)



from pathlib import Path

from menpodetect.paths import models_dir_path
from menpo.shape import bounding_box


# Paths to the OpenCV shipped with menpodetect
opencv_models_path = Path(models_dir_path(), 'opencv')
opencv_frontal_face_path = Path(opencv_models_path,
                                'haarcascade_frontalface_alt.xml')
opencv_profile_face_path = Path(opencv_models_path,
                                'haarcascade_profileface.xml')
opencv_eye_path = Path(opencv_models_path,
                       'haarcascade_eye.xml')


def pointgraph_from_rect(rect):
    r"""
    Convert an opencv detection to a menpo.shape.PointDirectedGraph.
    This enforces a particular point ordering.

    Parameters
    ----------
    rect : `tuple`
        The bounding box to convert. Result of calling an opencv detection.

    Returns
    -------
    bounding_box : `menpo.shape.PointDirectedGraph`
        A menpo PointDirectedGraph giving the bounding box.
    """
    x, y, w, h = rect
    return bounding_box((y, x), (y + h, x + w)
)



from menpodetect.bob import *
from menpodetect.dlib import *
from menpodetect.opencv import *
from menpodetect.pico import *
from menpodetect.ffld2 import *

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions



from menpo.base import MenpoMissingDependencyError


try:
    from .detect import FFLD2Detector, load_ffld2_frontal_face_detector
    from .train import train_ffld2_detector
except MenpoMissingDependencyError:
    pass

# Remove from scope
del MenpoMissingDependencyError
